import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

/**
 * Diese Klasse repraesentiert das Hauptfenster der Rechnungen-Hausaufgabe, wobei
 * eine eigene verkettete Liste verwendet wird.
 * 
 * @author Annabelle Klarl
 * 
 */
public class MeinRechnungsFrame extends JFrame implements ActionListener {
    private JButton rechnungHinzufuegenButton;
    private JButton teuersteRechnungBerechnenButton;

    private JTextArea ausgabeBereich;

    private MeineRechnungsListe rechnungsListe;

    /**
     * In diesem Programmstueck wird das Fenster erzeugt
     */
    public MeinRechnungsFrame() {
        /* Verbindung zum Modell der GUI */
        this.rechnungsListe = new MeineRechnungsListe();

        /* In der Kopfleiste des Fenster steht "MeinRechnungsFrame" */
        this.setTitle("MeinRechnungsFrame");

        /* Das Fenster ist 500 Pixel breit und 250 Pixel hoch. */
        this.setSize(500, 250);

        /* Hier werden alle Buttons erzeugt. */
        this.rechnungHinzufuegenButton = new JButton(
                "Neue Rechnung hinzufuegen");
        this.teuersteRechnungBerechnenButton = new JButton(
                "Teuerste Rechnungen anzeigen");

        /* Hier wird der Ausgabe-Bereich erzeugt. */
        this.ausgabeBereich = new JTextArea(10, 100);
        JScrollPane scrollPane = new JScrollPane(this.ausgabeBereich);
        scrollPane.setHorizontalScrollBarPolicy(
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);

        /*
         * Der ContentPane ist der Ausschnitt des Fensters, auf dem Widgets d.h.
         * Interaktionselemente (wie eine TextArea oder ein Button) platziert
         * werden koennen.
         */
        Container contentPane = this.getContentPane();
        contentPane.setLayout(new GridLayout(3, 1));
        /* Hier wird der Button platziert. */
        contentPane.add(this.rechnungHinzufuegenButton);
        contentPane.add(this.teuersteRechnungBerechnenButton);
        /* Hier wird der Ausgabebereich platziert. */
        contentPane.add(scrollPane);

        /*
         * Hier wird der Frame als Listener fuer Knopfdruck-Ereignisse bei jedem
         * der Buttons registriert.
         */
        this.rechnungHinzufuegenButton.addActionListener(this);
        this.teuersteRechnungBerechnenButton.addActionListener(this);

        /*
         * Wird das Fenster geschlossen (d.h. auf X gedrueckt), wird mit Hilfe
         * dieser Programmzeile auch unser Programm ordnungsgemaess beendet.
         */
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        // je nach Button entsprechende Methode ausfuehren
        Object source = e.getSource();
        if (source == this.rechnungHinzufuegenButton) {
            this.rechnungHinzufuegen();
        } else if (source == this.teuersteRechnungBerechnenButton) {
            this.teuersteRechnungBerechnen();
        }
    }

    /**
     * implementiert die Funktionalitaet des "Rechnung hinzufuegen"-Buttons
     */
    private void rechnungHinzufuegen() {
        String einlesenRechnungsnr = JOptionPane
                .showInputDialog("Rechnungsnummer: ");
        int rechnungsnr = Integer.parseInt(einlesenRechnungsnr);

        String einlesenBetrag = JOptionPane
                .showInputDialog("Rechnungsbetrag in Euro: ");
        int betrag = Integer.parseInt(einlesenBetrag);

        Rechnung neueRechnung = new Rechnung(rechnungsnr, betrag);
        this.rechnungsListe.addFirst(neueRechnung);

        this.ausgabeBereich.setText("Die Rechnung mit der Nummer "
                + rechnungsnr + " mit einem Betrag von " + betrag
                + " Euro wurde hinzugefuegt.");
    }

    /**
     * implementiert die Funktionalitaet des
     * "Teuerste Rechnung berechnen"-Buttons
     */
    private void teuersteRechnungBerechnen() {
        int hoechsterBetrag = -1;
        MeineRechnungsListe teuersteRechnungenListe = new MeineRechnungsListe();

        for (int i = 0; i < this.rechnungsListe.size(); i++) {
            Rechnung rechnung = this.rechnungsListe.get(i);
            int betrag = rechnung.getBetrag();

            if (betrag > hoechsterBetrag) {
                hoechsterBetrag = betrag;
                teuersteRechnungenListe = new MeineRechnungsListe();
                teuersteRechnungenListe.addFirst(rechnung);
            } else if (betrag == hoechsterBetrag) {
                teuersteRechnungenListe.addFirst(rechnung);
            }
            // else: diese Rechnung ist billiger als die aktuell teuerste Rechnung
        }

        this.ausgabeBereich
                .setText("Die teuersten Rechnungen sind: \n");
        for (int i = 0; i < teuersteRechnungenListe.size(); i++) {
            Rechnung rechnung = teuersteRechnungenListe.get(i);
            this.ausgabeBereich.append(rechnung.getRechnungsnr()
                    + " mit einem Betrag von " + rechnung.getBetrag()
                    + " Euro");
            this.ausgabeBereich.append("\n");
        }
    }
}
