import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

/**
 * Diese Klasse repraesentiert das Hauptfenster der Postkarten-Praesenzaufgabe,
 * wobei {@link LinkedList} verwendet wird.
 * 
 * @author Annabelle Klarl
 * 
 */
public class PostkartenFrame extends JFrame implements ActionListener {
    private JButton postkarteHinzufuegenButton;
    private JButton siegerBerechnenButton;

    private JTextArea ausgabeBereich;

    private LinkedList<Postkarte> postkartenListe;

    /**
     * In diesem Programmstueck wird das Fenster erzeugt
     */
    public PostkartenFrame() {
        /* Verbindung zum Modell der GUI */
        this.postkartenListe = new LinkedList<Postkarte>();

        /* In der Kopfleiste des Fenster steht "PostkartenFrame" */
        this.setTitle("PostkartenFrame");

        /* Das Fenster ist 500 Pixel breit und 250 Pixel hoch. */
        this.setSize(500, 250);

        /* Hier werden alle Buttons erzeugt. */
        this.postkarteHinzufuegenButton = new JButton(
                "Neue Postkarte hinzufuegen");
        this.siegerBerechnenButton = new JButton(
                "Sieger aus allen bisherigen Postkarten berechnen");

        /* Hier wird der Ausgabe-Bereich erzeugt. */
        this.ausgabeBereich = new JTextArea(10, 100);
        JScrollPane scrollPane = new JScrollPane(this.ausgabeBereich);
        scrollPane.setHorizontalScrollBarPolicy(
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);

        /*
         * Der ContentPane ist der Ausschnitt des Fensters, auf dem Widgets
         * d.h. Interaktionselemente (wie eine TextArea oder ein Button)
         * platziert werden koennen.
         */
        Container contentPane = this.getContentPane();
        contentPane.setLayout(new GridLayout(3, 1));
        /* Hier wird der Button platziert. */
        contentPane.add(this.postkarteHinzufuegenButton);
        contentPane.add(this.siegerBerechnenButton);
        /* Hier wird der Ausgabebereich platziert. */
        contentPane.add(scrollPane);

        /*
         * Hier wird der Frame als Listener fuer Knopfdruck-Ereignisse bei
         * jedem der Buttons registriert.
         */
        this.postkarteHinzufuegenButton.addActionListener(this);
        this.siegerBerechnenButton.addActionListener(this);

        /*
         * Wird das Fenster geschlossen (d.h. auf X gedrueckt), wird mit Hilfe
         * dieser Programmzeile auch unser Programm ordnungsgemaess beendet.
         */
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        // je nach Button entsprechende Methode ausfuehren
        Object source = e.getSource();
        if (source == this.postkarteHinzufuegenButton) {
            this.postkarteHinzufuegen();
        } else if (source == this.siegerBerechnenButton) {
            this.siegerBerechnen();
        }
    }

    /**
     * implementiert die Funktionalitaet des "Postkarte hinzufuegen"-Buttons
     */
    private void postkarteHinzufuegen() {
        String name = JOptionPane.showInputDialog(
                "Name des Teilnehmers: ");

        String einlesenEntfernung = JOptionPane.showInputDialog(
                "Zurueckgelegte Entfernung in km: ");
        int entfernung = Integer.parseInt(einlesenEntfernung);

        Postkarte neuePostkarte = new Postkarte(name, entfernung);
        this.postkartenListe.addFirst(neuePostkarte);

        this.ausgabeBereich.setText("Die Postkarte des Teilnehmers "
                + name + " mit einer Entfernung von " + entfernung
                + " km wurde hinzugefuegt.");
    }

    /**
     * implementiert die Funktionalitaet des "Sieger berechnen"-Buttons
     */
    private void siegerBerechnen() {
        int groessteEntfernung = -1;
        LinkedList<Postkarte> siegerListe = new LinkedList<Postkarte>();

        Iterator<Postkarte> iteratorPostkarten = this.postkartenListe.iterator();
        while (iteratorPostkarten.hasNext()) {
            Postkarte postkarte = iteratorPostkarten.next();
            int entfernung = postkarte.getEntfernung();

            if (entfernung > groessteEntfernung) {
                groessteEntfernung = entfernung;
                siegerListe = new LinkedList<Postkarte>();
                siegerListe.addFirst(postkarte);
            } else if (entfernung == groessteEntfernung) {
                siegerListe.addFirst(postkarte);
            }
            // else: diese Postkarte ist nicht weiter gereist
            // als die bisherigen Siegerpostkarten
        }

        this.ausgabeBereich.setText("Die Sieger sind: \n");
        Iterator<Postkarte> iteratorGewinner = siegerListe.iterator();
        while (iteratorGewinner.hasNext()) {
            Postkarte postkarte = iteratorGewinner.next();
            this.ausgabeBereich.append(postkarte.getTeilnehmer()
                    + " mit einer Entfernung von "
                    + postkarte.getEntfernung() + " km");
            this.ausgabeBereich.append("\n");
        }
    }
}
